import os
from unittest import mock

from dojo_toolkit.settings import ASSETS_DIR

# workaround to tests run on travis
try:
    import gi

    gi.require_version("Notify", "0.7")
    from gi.repository import GdkPixbuf, Notify
except ImportError:
    Notify = mock.Mock()
    GdkPixbuf = mock.Mock()


class BaseNotifier:
    def __init__(self):
        self.fail_img_path = os.path.join(ASSETS_DIR, "r.jpg")
        self.success_img_path = os.path.join(ASSETS_DIR, "g.jpg")

    def get_notifier(self):
        raise NotImplementedError()

    def notify(self, message, image=None):
        raise NotImplementedError()


class GnomeNotifier(BaseNotifier):
    def __init__(self):
        super(GnomeNotifier, self).__init__()

        self.fail_img = GdkPixbuf.Pixbuf.new_from_file(self.fail_img_path)
        self.success_img = GdkPixbuf.Pixbuf.new_from_file(self.success_img_path)

        self._notifier = self.get_notifier()

    def get_notifier(self):
        Notify.init("not")
        return Notify.Notification.new("", "", "")

    def notify(self, message, image_path="", timeout=5 * 60 * 1000):
        self._notifier.update(message, "", image_path)
        self._notifier.set_timeout(timeout)
        self._notifier.show()

    def success(self, message):
        self.notify(message, self.success_img_path)

    def fail(self, message):
        self.notify(message, self.fail_img_path)


notifier = GnomeNotifier()



import os
from argparse import ArgumentParser

from dojo_toolkit.dojo import Dojo


def parse_args():
    """
    build argument parser
    """
    parser = ArgumentParser()
    parser.add_argument(
        "-t",
        "--time",
        help="the amount of time a dojo round lasts",
        default=Dojo.ROUND_TIME,
        type=float,
    )
    parser.add_argument(
        "path",
        help="the path to the folder containing the code used during the dojo",
        nargs="?",
        default=".",
        type=os.path.realpath,
    )
    parser.add_argument(
        "--mute",
        help="mute all sounds (default: all sounds are played), note: this only works in Linux",
        action="store_true",
    )
    parser.add_argument(
        "--runner", help="name of the runner (default: doctest)", default="doctest", type=str
    )

    return parser.parse_args()


def main():
    args = parse_args()
    dojo = Dojo(code_path=args.path, round_time=args.time, mute=args.mute, runner=args.runner)
    dojo.start()



import os

import pyglet

from dojo_toolkit.settings import SOUNDS_DIR


class SoundHandler:
    def __init__(self):
        self.player = pyglet.media.Player()

        start_audio_path = os.path.join(SOUNDS_DIR, "start.wav")
        self.start_media = pyglet.media.load(start_audio_path, streaming=False)

        success_audio_path = os.path.join(SOUNDS_DIR, "pass.wav")
        self.success_media = pyglet.media.load(success_audio_path, streaming=False)

        timeup_audio_path = os.path.join(SOUNDS_DIR, "timeup.wav")
        self.timeup_media = pyglet.media.load(timeup_audio_path, streaming=False)

    def play(self):
        if self.player.playing:
            self.player.next_source()
        else:
            self.player.play()

    def play_start(self):
        self.player.queue(self.start_media)
        self.play()

    def play_success(self):
        self.player.queue(self.success_media)
        self.play()

    def play_timeup(self):
        self.player.queue(self.timeup_media)
        self.play()



import time

from watchdog.events import PatternMatchingEventHandler

from dojo_toolkit.notifier import notifier


class DojoCodeHandler(PatternMatchingEventHandler):
    """Handles the python file changes"""

    patterns = ["*.py"]
    ignore_directories = True
    min_test_time_interval = 2

    def __init__(self, *args, **kwargs):
        self.dojo = kwargs.pop("dojo")
        self.test_runner = kwargs.pop("test_runner")

        super(DojoCodeHandler, self).__init__(*args, **kwargs)

        self.last_test_run_time = time.time()

    def get_last_test_run_interval(self):
        return time.time() - self.last_test_run_time

    def handle_stopped_round(self):
        notifier.notify("Round has not been started")
        print("Press <Enter> to start the round")

    def on_modified(self, event):
        """Called when a file in the dojo directory is modified
        runs the doctest and display a notification
        Green for 'ok to talk, test passing'
        Red for 'not ok to talk, test failing'
        """

        if not self.dojo.round_started:
            return self.handle_stopped_round()

        if self.get_last_test_run_interval() < self.min_test_time_interval:
            return
        self.last_test_run_time = time.time()

        self.test_runner.run()



import sys
import time
from threading import Thread
from unittest import mock

from clint.textui import colored
from watchdog.observers import Observer

from dojo_toolkit.code_handler import DojoCodeHandler
from dojo_toolkit.notifier import notifier
from dojo_toolkit.sound_handler import SoundHandler
from dojo_toolkit.test_runner import get_test_runner
from dojo_toolkit.timer import Timer


class Dojo:
    ROUND_TIME = 5
    round_started = False

    def __init__(self, code_path, round_time=None, mute=False, test_runner=None, runner="doctest"):
        self.code_path = code_path
        self.round_time = round_time or self.ROUND_TIME
        self.sound_player = mock.Mock() if mute or sys.platform != "linux" else SoundHandler()
        self.info_notified = False

        test_runner = get_test_runner(test_runner, runner, self.code_path, self.sound_player)

        event_handler = DojoCodeHandler(dojo=self, test_runner=test_runner)

        self.observer = Observer()
        self.observer.schedule(event_handler, self.code_path, recursive=False)

        self.timer = Timer(self.round_time)

    def start(self):
        self.observer.start()
        print("\nWatching: {} folder".format(self.code_path))

        self.is_running = True
        print("Dojo toolkit started!")
        self.thread = Thread(target=self.dojo)
        self.thread.start()

        self.thread.join()
        self.observer.join()

    def stop(self):
        self.is_running = False

    def await_pilot_exchange(self):
        print("Awaiting the pilot and co-pilot to enter their positions.")
        print("Press <Enter> when they are ready")
        input()

    def round_start(self):
        self.timer.start()
        self.sound_player.play_start()
        self.round_started = True
        self.info_notified = False

        print("Round started! {} minutes left...".format(self.round_time))

    def round_info(self):
        if self.timer.ellapsed_time == self.timer.duration - 60 and not self.info_notified:
            notifier.notify("60 seconds to round finish...")
            print((getattr(colored, "yellow"))("Round is going to finish in 60 seconds"))
            self.info_notified = True

    def round_finished(self):
        notifier.notify("Time Up", timeout=15 * 1000)
        self.sound_player.play_timeup()
        self.round_started = False
        print("Round finished!\n")

    def dojo(self):
        while self.is_running:
            self.await_pilot_exchange()
            self.round_start()
            while self.timer.is_running:
                self.round_info()
                time.sleep(0.8)
            self.round_finished()



import time
from math import floor
from threading import Thread


class Timer:
    """
    A timer that runs on a separated thread and "ticks" every second. It
    keeps track of the ellapsed time.
    """

    def __init__(self, duration_in_minutes):
        self.duration = duration_in_minutes * 60

    def start(self):
        self.ellapsed_time = 0

        self.is_running = True
        self.start_time = time.time()

        self.thread = Thread(target=self.timer)
        self.thread.start()

    def timer(self):
        while self.ellapsed_time <= self.duration:
            self.ellapsed_time = floor(time.time() - self.start_time)
            time.sleep(1)
        self.is_running = False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dojo_toolkit']

package_data = \
{'': ['*'], 'dojo_toolkit': ['assets/*', 'assets/sounds/*']}

install_requires = \
['clint>=0.5.1,<0.6.0', 'pyglet>=1.5.21,<2.0.0', 'watchdog>=2.1.6,<3.0.0']

extras_require = \
{':sys_platform == "linux"': ['pgi>=0.0.11,<0.0.12']}

entry_points = \
{'console_scripts': ['dojo = dojo_toolkit.main:main']}

setup_kwargs = {
    'name': 'dojo-toolkit',
    'version': '0.6.0',
    'description': 'Toolkit for Python Coding Dojos.',
    'long_description': None,
    'author': 'grupy-sanca',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""
Module for running tests like doctest and unittest
"""
import os
import subprocess

from clint.textui import colored

from dojo_toolkit.notifier import notifier


def get_test_runner(test_runner, runner, code_path, sound_player):
    if test_runner:
        return test_runner
    if runner == "doctest":
        return DoctestTestRunner(code_path, sound_player)
    elif runner == "pytest":
        return PytestTestRunner(code_path, sound_player)
    else:
        raise NotImplementedError("Invalid runner")


class LocalTestRunner:
    def __init__(self, code_path, sound_player):
        self.code_path = code_path
        self.sound_player = sound_player

    def _run_test(self):
        raise NotImplementedError("_run_test() must be implemented")

    def run(self):
        result = self._run_test()
        self._clear_screen()
        self._handle_result(result["is_success"])
        print(result["output"])
        return result["is_success"]

    def _clear_screen(self):
        command = "cls" if os.name == "nt" else "clear"
        subprocess.call(command, shell=True)

    def _handle_result(self, success):
        if success:
            self._handle_success()
        else:
            self._handle_failure()

    def _handle_success(self):
        print(getattr(colored, "green")("\nTests passed!\n"))
        notifier.success("OK TO TALK")
        self.sound_player.play_success()

    def _handle_failure(self):
        print(getattr(colored, "red")("\nTests failed!\n"))
        notifier.fail("NOT OK TO TALK")


class DoctestTestRunner(LocalTestRunner):
    def _run_test(self):
        result = subprocess.run(
            ["python -m doctest " + self.code_path + "/*.py"],
            capture_output=True,
            shell=True,
            encoding="utf-8",
        )

        return {
            "is_success": result.returncode == 0,
            "output": result.stdout,
        }


class PytestTestRunner(LocalTestRunner):
    def _run_test(self):
        result = subprocess.run(
            ["python -m pytest " + self.code_path + "/*.py"],
            capture_output=True,
            shell=True,
            encoding="utf-8",
        )

        return {
            "is_success": result.returncode == 0,
            "output": result.stdout,
        }
